searchData={"items":[{"type":"module","title":"MCPEx","doc":"MCPEx is an Elixir client library for the Model Context Protocol (MCP).\n\nThis library provides a complete, spec-compliant implementation of the\nModel Context Protocol (MCP) client, supporting version 2025-03-26 of the specification.","ref":"MCPEx.html"},{"type":"module","title":"Features - MCPEx","doc":"* Full protocol implementation with capability negotiation\n* Multiple transport options (stdio, HTTP)\n* Resource handling with subscriptions\n* Tool invocation support\n* Prompt template handling\n* Shell environment integration","ref":"MCPEx.html#module-features"},{"type":"function","title":"MCPEx.protocol_version/0","doc":"Returns the current MCP protocol version supported by this library.","ref":"MCPEx.html#protocol_version/0"},{"type":"module","title":"MCPEx.Client","doc":"An implementation of the MCP client for Elixir.\n\nThis client provides a complete, spec-compliant implementation of the Model Context Protocol\n(MCP) client side, supporting version 2025-03-26 of the specification.","ref":"MCPEx.Client.html"},{"type":"module","title":"Features - MCPEx.Client","doc":"* Full protocol implementation with capability negotiation\n* Multiple transport options (stdio, HTTP)\n* Resource handling with subscriptions\n* Tool invocation support\n* Prompt template handling\n* Sampling interface for LLM interaction","ref":"MCPEx.Client.html#module-features"},{"type":"module","title":"Usage - MCPEx.Client","doc":"```elixir\n# Create a new client connected to a server via stdio\n{:ok, client} = MCPEx.Client.start_link(\n  transport: :stdio,\n  command: \"/path/to/server\",\n  capabilities: [:sampling, :roots]\n)\n\n# Or with HTTP transport\n{:ok, client} = MCPEx.Client.start_link(\n  transport: :http,\n  url: \"https://example.com/mcp\",\n  capabilities: [:sampling, :roots]\n)\n\n# List available resources\n{:ok, %{resources: resources}} = MCPEx.Client.list_resources(client)\n\n# Read a specific resource\n{:ok, %{contents: contents}} = MCPEx.Client.read_resource(client, \"file:///path/to/resource\")\n\n# Call a tool\n{:ok, result} = MCPEx.Client.call_tool(client, \"get_weather\", %{location: \"New York\"})\n```","ref":"MCPEx.Client.html#module-usage"},{"type":"function","title":"MCPEx.Client.call_tool/3","doc":"Calls a tool on the server.","ref":"MCPEx.Client.html#call_tool/3"},{"type":"function","title":"Parameters - MCPEx.Client.call_tool/3","doc":"* `name` - The name of the tool to call\n* `args` - The arguments to pass to the tool","ref":"MCPEx.Client.html#call_tool/3-parameters"},{"type":"function","title":"Returns - MCPEx.Client.call_tool/3","doc":"* `{:ok, result}` - The result of the tool call\n* `{:error, reason}` - Failed to call the tool","ref":"MCPEx.Client.html#call_tool/3-returns"},{"type":"function","title":"MCPEx.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MCPEx.Client.html#child_spec/1"},{"type":"function","title":"MCPEx.Client.get_prompt/3","doc":"Gets a specific prompt by name with optional parameters.","ref":"MCPEx.Client.html#get_prompt/3"},{"type":"function","title":"Parameters - MCPEx.Client.get_prompt/3","doc":"* `name` - The name of the prompt to get\n* `args` - Optional arguments to apply to the prompt template","ref":"MCPEx.Client.html#get_prompt/3-parameters"},{"type":"function","title":"Returns - MCPEx.Client.get_prompt/3","doc":"* `{:ok, %{description: String.t(), messages: [...]}}` - Successfully fetched prompt with messages\n* `{:error, reason}` - Failed to fetch prompt","ref":"MCPEx.Client.html#get_prompt/3-returns"},{"type":"function","title":"MCPEx.Client.get_server_capabilities/1","doc":"Returns the server capabilities negotiated during initialization.","ref":"MCPEx.Client.html#get_server_capabilities/1"},{"type":"function","title":"Returns - MCPEx.Client.get_server_capabilities/1","doc":"* `{:ok, map()}` - Server capabilities\n* `{:error, reason}` - Failed to get capabilities","ref":"MCPEx.Client.html#get_server_capabilities/1-returns"},{"type":"function","title":"MCPEx.Client.get_server_info/1","doc":"Returns information about the connected server.","ref":"MCPEx.Client.html#get_server_info/1"},{"type":"function","title":"Returns - MCPEx.Client.get_server_info/1","doc":"* `{:ok, map()}` - Server information\n* `{:error, reason}` - Failed to get server information","ref":"MCPEx.Client.html#get_server_info/1-returns"},{"type":"function","title":"MCPEx.Client.list_prompts/2","doc":"Lists available prompts from the server.","ref":"MCPEx.Client.html#list_prompts/2"},{"type":"function","title":"Parameters - MCPEx.Client.list_prompts/2","doc":"* `cursor` - An optional pagination cursor for fetching the next page of results","ref":"MCPEx.Client.html#list_prompts/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.list_prompts/2","doc":"* `{:ok, %{prompts: [...], next_cursor: cursor}}` - Successfully fetched prompts\n* `{:error, reason}` - Failed to fetch prompts","ref":"MCPEx.Client.html#list_prompts/2-returns"},{"type":"function","title":"MCPEx.Client.list_resource_templates/2","doc":"Lists available resource templates from the server.","ref":"MCPEx.Client.html#list_resource_templates/2"},{"type":"function","title":"Parameters - MCPEx.Client.list_resource_templates/2","doc":"* `cursor` - An optional pagination cursor for fetching the next page of results","ref":"MCPEx.Client.html#list_resource_templates/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.list_resource_templates/2","doc":"* `{:ok, %{resource_templates: [...], next_cursor: cursor}}` - Successfully fetched templates\n* `{:error, reason}` - Failed to fetch templates","ref":"MCPEx.Client.html#list_resource_templates/2-returns"},{"type":"function","title":"MCPEx.Client.list_resources/2","doc":"Lists available resources from the server.","ref":"MCPEx.Client.html#list_resources/2"},{"type":"function","title":"Parameters - MCPEx.Client.list_resources/2","doc":"* `cursor` - An optional pagination cursor for fetching the next page of results","ref":"MCPEx.Client.html#list_resources/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.list_resources/2","doc":"* `{:ok, %{resources: [...], next_cursor: cursor}}` - Successfully fetched resources\n* `{:error, reason}` - Failed to fetch resources","ref":"MCPEx.Client.html#list_resources/2-returns"},{"type":"function","title":"MCPEx.Client.list_tools/2","doc":"Lists available tools from the server.","ref":"MCPEx.Client.html#list_tools/2"},{"type":"function","title":"Parameters - MCPEx.Client.list_tools/2","doc":"* `cursor` - An optional pagination cursor for fetching the next page of results","ref":"MCPEx.Client.html#list_tools/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.list_tools/2","doc":"* `{:ok, %{tools: [...], next_cursor: cursor}}` - Successfully fetched tools\n* `{:error, reason}` - Failed to fetch tools","ref":"MCPEx.Client.html#list_tools/2-returns"},{"type":"type","title":"MCPEx.Client.options/0","doc":"Client options for initialization","ref":"MCPEx.Client.html#t:options/0"},{"type":"function","title":"MCPEx.Client.ping/2","doc":"Sends a ping to the server to check if it's still alive.","ref":"MCPEx.Client.html#ping/2"},{"type":"function","title":"Parameters - MCPEx.Client.ping/2","doc":"* `client` - The client process\n* `timeout` - Optional timeout in milliseconds (default: 5000)","ref":"MCPEx.Client.html#ping/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.ping/2","doc":"* `{:ok, _}` - The server responded to the ping\n* `{:error, reason}` - The ping failed","ref":"MCPEx.Client.html#ping/2-returns"},{"type":"function","title":"MCPEx.Client.read_resource/2","doc":"Reads the contents of a specific resource from the server.","ref":"MCPEx.Client.html#read_resource/2"},{"type":"function","title":"Parameters - MCPEx.Client.read_resource/2","doc":"* `uri` - The URI of the resource to read","ref":"MCPEx.Client.html#read_resource/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.read_resource/2","doc":"* `{:ok, %{contents: [...]}}` - Successfully read the resource\n* `{:error, reason}` - Failed to read the resource","ref":"MCPEx.Client.html#read_resource/2-returns"},{"type":"function","title":"MCPEx.Client.start_link/1","doc":"Starts a new MCP client as a linked process.","ref":"MCPEx.Client.html#start_link/1"},{"type":"function","title":"Options - MCPEx.Client.start_link/1","doc":"* `:transport` - The transport mechanism to use (`:stdio` or `:http`)\n* `:command` - The command to run for stdio transport (required for stdio)\n* `:url` - The URL to connect to for HTTP transport (required for http)\n* `:capabilities` - A list of capabilities to advertise to the server\n* `:client_info` - Information about the client implementation (name, version)","ref":"MCPEx.Client.html#start_link/1-options"},{"type":"function","title":"Returns - MCPEx.Client.start_link/1","doc":"* `{:ok, pid}` - The client was started successfully\n* `{:error, reason}` - The client failed to start","ref":"MCPEx.Client.html#start_link/1-returns"},{"type":"function","title":"MCPEx.Client.stop/1","doc":"Stops the client and terminates the connection.","ref":"MCPEx.Client.html#stop/1"},{"type":"function","title":"Returns - MCPEx.Client.stop/1","doc":"* `:ok` - The client was stopped successfully","ref":"MCPEx.Client.html#stop/1-returns"},{"type":"function","title":"MCPEx.Client.subscribe_to_resource/2","doc":"Subscribes to updates for a specific resource.","ref":"MCPEx.Client.html#subscribe_to_resource/2"},{"type":"function","title":"Parameters - MCPEx.Client.subscribe_to_resource/2","doc":"* `uri` - The URI of the resource to subscribe to","ref":"MCPEx.Client.html#subscribe_to_resource/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.subscribe_to_resource/2","doc":"* `:ok` - Successfully subscribed\n* `{:error, reason}` - Failed to subscribe","ref":"MCPEx.Client.html#subscribe_to_resource/2-returns"},{"type":"type","title":"MCPEx.Client.t/0","doc":"Client state","ref":"MCPEx.Client.html#t:t/0"},{"type":"function","title":"MCPEx.Client.unsubscribe_from_resource/2","doc":"Unsubscribes from updates for a specific resource.","ref":"MCPEx.Client.html#unsubscribe_from_resource/2"},{"type":"function","title":"Parameters - MCPEx.Client.unsubscribe_from_resource/2","doc":"* `uri` - The URI of the resource to unsubscribe from","ref":"MCPEx.Client.html#unsubscribe_from_resource/2-parameters"},{"type":"function","title":"Returns - MCPEx.Client.unsubscribe_from_resource/2","doc":"* `:ok` - Successfully unsubscribed\n* `{:error, reason}` - Failed to unsubscribe","ref":"MCPEx.Client.html#unsubscribe_from_resource/2-returns"},{"type":"module","title":"MCPEx.Protocol.Capabilities","doc":"Handles protocol capabilities and version negotiation.\n\nThis module provides utilities for:\n- Creating client and server capability maps\n- Checking for capability support\n- Negotiating protocol versions\n- Validating capability requirements\n\nThe MCP protocol uses capabilities to allow clients and servers to \ndeclare which features they support, with both required and optional\ncapabilities.","ref":"MCPEx.Protocol.Capabilities.html"},{"type":"function","title":"MCPEx.Protocol.Capabilities.build_client_capabilities/1","doc":"Builds client capabilities map based on provided capabilities list. \nDEPRECATED: Use create_client_capabilities/1 instead.","ref":"MCPEx.Protocol.Capabilities.html#build_client_capabilities/1"},{"type":"function","title":"MCPEx.Protocol.Capabilities.create_client_capabilities/1","doc":"Creates a client capabilities object according to the MCP specification.","ref":"MCPEx.Protocol.Capabilities.html#create_client_capabilities/1"},{"type":"function","title":"Options - MCPEx.Protocol.Capabilities.create_client_capabilities/1","doc":"* `sampling` - Whether to include sampling capability (default: true)\n* `roots` - Whether to include roots capability (default: false)\n* `roots_list_changed` - Whether roots supports list change notifications (default: false)\n* `experimental` - Optional map of experimental capabilities","ref":"MCPEx.Protocol.Capabilities.html#create_client_capabilities/1-options"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.create_client_capabilities/1","doc":"* `map()` - The client capabilities object ready for initialization","ref":"MCPEx.Protocol.Capabilities.html#create_client_capabilities/1-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Capabilities.create_client_capabilities/1","doc":"iex> MCPEx.Protocol.Capabilities.create_client_capabilities()\n    %{sampling: %{}}\n    \n    iex> MCPEx.Protocol.Capabilities.create_client_capabilities(roots: true, roots_list_changed: true)\n    %{sampling: %{}, roots: %{listChanged: true}}","ref":"MCPEx.Protocol.Capabilities.html#create_client_capabilities/1-examples"},{"type":"function","title":"MCPEx.Protocol.Capabilities.create_server_capabilities/1","doc":"Creates a server capabilities object according to the MCP specification.","ref":"MCPEx.Protocol.Capabilities.html#create_server_capabilities/1"},{"type":"function","title":"Options - MCPEx.Protocol.Capabilities.create_server_capabilities/1","doc":"* `resources` - Whether to include resources capability (default: false)\n* `resources_subscribe` - Whether resources can be subscribed to (default: false)\n* `resources_list_changed` - Whether resources supports list change notifications (default: false)\n* `tools` - Whether to include tools capability (default: false)\n* `tools_list_changed` - Whether tools supports list change notifications (default: false)\n* `prompts` - Whether to include prompts capability (default: false)\n* `prompts_list_changed` - Whether prompts supports list change notifications (default: false)\n* `logging` - Whether to include logging capability (default: false)\n* `experimental` - Optional map of experimental capabilities","ref":"MCPEx.Protocol.Capabilities.html#create_server_capabilities/1-options"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.create_server_capabilities/1","doc":"* `map()` - The server capabilities object","ref":"MCPEx.Protocol.Capabilities.html#create_server_capabilities/1-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Capabilities.create_server_capabilities/1","doc":"iex> MCPEx.Protocol.Capabilities.create_server_capabilities(resources: true, resources_subscribe: true)\n    %{resources: %{subscribe: true}}","ref":"MCPEx.Protocol.Capabilities.html#create_server_capabilities/1-examples"},{"type":"function","title":"MCPEx.Protocol.Capabilities.has_capability?/3","doc":"Checks if a specific capability is available.","ref":"MCPEx.Protocol.Capabilities.html#has_capability?/3"},{"type":"function","title":"Parameters - MCPEx.Protocol.Capabilities.has_capability?/3","doc":"* `capabilities` - The capability map to check\n* `capability` - The main capability to check for\n* `sub_capability` - Optional sub-capability to check for","ref":"MCPEx.Protocol.Capabilities.html#has_capability?/3-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.has_capability?/3","doc":"* `boolean()` - `true` if the capability is available, otherwise `false`","ref":"MCPEx.Protocol.Capabilities.html#has_capability?/3-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Capabilities.has_capability?/3","doc":"iex> capabilities = %{resources: %{subscribe: true}}\n    iex> MCPEx.Protocol.Capabilities.has_capability?(capabilities, :resources)\n    true\n    \n    iex> capabilities = %{resources: %{subscribe: true}}\n    iex> MCPEx.Protocol.Capabilities.has_capability?(capabilities, :resources, :subscribe)\n    true\n    \n    iex> capabilities = %{resources: %{subscribe: true}}\n    iex> MCPEx.Protocol.Capabilities.has_capability?(capabilities, :sampling)\n    false","ref":"MCPEx.Protocol.Capabilities.html#has_capability?/3-examples"},{"type":"function","title":"MCPEx.Protocol.Capabilities.latest_version/0","doc":"Gets the latest supported protocol version.","ref":"MCPEx.Protocol.Capabilities.html#latest_version/0"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.latest_version/0","doc":"* `String.t()` - The latest supported version","ref":"MCPEx.Protocol.Capabilities.html#latest_version/0-returns"},{"type":"function","title":"MCPEx.Protocol.Capabilities.negotiate_version/2","doc":"Negotiates a common protocol version between client and server.","ref":"MCPEx.Protocol.Capabilities.html#negotiate_version/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.Capabilities.negotiate_version/2","doc":"* `client_versions` - Client supported version(s)\n* `server_versions` - Server supported version(s)","ref":"MCPEx.Protocol.Capabilities.html#negotiate_version/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.negotiate_version/2","doc":"* `{:ok, version}` - The negotiated common version\n* `{:error, reason}` - Error when no common version exists","ref":"MCPEx.Protocol.Capabilities.html#negotiate_version/2-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Capabilities.negotiate_version/2","doc":"iex> client_version = \"2025-03-26\"\n    iex> server_versions = [\"2025-03-26\", \"2024-11-05\"]\n    iex> MCPEx.Protocol.Capabilities.negotiate_version(client_version, server_versions)\n    {:ok, \"2025-03-26\"}","ref":"MCPEx.Protocol.Capabilities.html#negotiate_version/2-examples"},{"type":"function","title":"MCPEx.Protocol.Capabilities.parse_server_capabilities/1","doc":"Parses server capabilities from an initialize response.","ref":"MCPEx.Protocol.Capabilities.html#parse_server_capabilities/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Capabilities.parse_server_capabilities/1","doc":"* `response` - The initialization response from the server","ref":"MCPEx.Protocol.Capabilities.html#parse_server_capabilities/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.parse_server_capabilities/1","doc":"* `{:ok, capabilities}` - The parsed server capabilities\n* `{:error, reason}` - Error parsing capabilities","ref":"MCPEx.Protocol.Capabilities.html#parse_server_capabilities/1-returns"},{"type":"function","title":"MCPEx.Protocol.Capabilities.protocol_version/0","doc":"Returns the current protocol version supported by this client.","ref":"MCPEx.Protocol.Capabilities.html#protocol_version/0"},{"type":"function","title":"MCPEx.Protocol.Capabilities.supported_versions/0","doc":"Gets all supported protocol versions.","ref":"MCPEx.Protocol.Capabilities.html#supported_versions/0"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.supported_versions/0","doc":"* `[String.t()]` - List of supported versions","ref":"MCPEx.Protocol.Capabilities.html#supported_versions/0-returns"},{"type":"function","title":"MCPEx.Protocol.Capabilities.validate_capability_support/2","doc":"Validates that required capabilities are supported.","ref":"MCPEx.Protocol.Capabilities.html#validate_capability_support/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.Capabilities.validate_capability_support/2","doc":"* `required_capabilities` - List of capability atoms that are required\n* `available_capabilities` - Map of capabilities that are available","ref":"MCPEx.Protocol.Capabilities.html#validate_capability_support/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Capabilities.validate_capability_support/2","doc":"* `:ok` - All required capabilities are available\n* `{:error, reason}` - Some required capabilities are missing","ref":"MCPEx.Protocol.Capabilities.html#validate_capability_support/2-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Capabilities.validate_capability_support/2","doc":"iex> MCPEx.Protocol.Capabilities.validate_capability_support(\n    ...>   [:sampling], \n    ...>   %{resources: %{}, sampling: %{}}\n    ...> )\n    :ok","ref":"MCPEx.Protocol.Capabilities.html#validate_capability_support/2-examples"},{"type":"module","title":"MCPEx.Protocol.Errors","doc":"Defines and handles MCP protocol errors.\n\nThis module provides utilities for creating and processing\nJSON-RPC error objects according to the MCP specification.","ref":"MCPEx.Protocol.Errors.html"},{"type":"module","title":"JSON-RPC Error Codes - MCPEx.Protocol.Errors","doc":"","ref":"MCPEx.Protocol.Errors.html#module-json-rpc-error-codes"},{"type":"module","title":"Pre-defined JSON-RPC 2.0 error codes: - MCPEx.Protocol.Errors","doc":"- `-32700`: Parse error - Invalid JSON was received\n- `-32600`: Invalid Request - The JSON sent is not a valid Request object\n- `-32601`: Method not found - The method does not exist / is not available\n- `-32602`: Invalid params - Invalid method parameter(s)\n- `-32603`: Internal error - Internal JSON-RPC error\n- `-32000 to -32099`: Server error - Reserved for implementation-defined server-errors","ref":"MCPEx.Protocol.Errors.html#module-pre-defined-json-rpc-2-0-error-codes"},{"type":"module","title":"MCP-specific error codes: - MCPEx.Protocol.Errors","doc":"- `-32800`: Request cancelled - The request was cancelled\n- `-32801`: Content too large - The content is too large","ref":"MCPEx.Protocol.Errors.html#module-mcp-specific-error-codes"},{"type":"function","title":"MCPEx.Protocol.Errors.content_too_large/1","doc":"Creates a content too large error object.","ref":"MCPEx.Protocol.Errors.html#content_too_large/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.content_too_large/1","doc":"* `details` - Details about the content size issues","ref":"MCPEx.Protocol.Errors.html#content_too_large/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.content_too_large/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#content_too_large/1-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.create_error/3","doc":"Creates a JSON-RPC error object.","ref":"MCPEx.Protocol.Errors.html#create_error/3"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.create_error/3","doc":"* `code` - The error code\n* `message` - The error message\n* `data` - Optional additional data","ref":"MCPEx.Protocol.Errors.html#create_error/3-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.create_error/3","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#create_error/3-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Errors.create_error/3","doc":"iex> MCPEx.Protocol.Errors.create_error(-32600, \"Invalid Request\")\n    %{code: -32600, message: \"Invalid Request\"}\n    \n    iex> MCPEx.Protocol.Errors.create_error(-32602, \"Missing param\", %{param: \"id\"})\n    %{code: -32602, message: \"Missing param\", data: %{param: \"id\"}}","ref":"MCPEx.Protocol.Errors.html#create_error/3-examples"},{"type":"function","title":"MCPEx.Protocol.Errors.encode_error_response/2","doc":"Creates a complete JSON-RPC error response.","ref":"MCPEx.Protocol.Errors.html#encode_error_response/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.encode_error_response/2","doc":"* `id` - The request ID (or nil for notifications)\n* `error` - The error object","ref":"MCPEx.Protocol.Errors.html#encode_error_response/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.encode_error_response/2","doc":"* `map()` - The complete error response","ref":"MCPEx.Protocol.Errors.html#encode_error_response/2-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.error_response_json/2","doc":"Returns a JSON string for an error response.","ref":"MCPEx.Protocol.Errors.html#error_response_json/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.error_response_json/2","doc":"* `id` - The request ID\n* `error` - The error object","ref":"MCPEx.Protocol.Errors.html#error_response_json/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.error_response_json/2","doc":"* `String.t()` - The JSON string","ref":"MCPEx.Protocol.Errors.html#error_response_json/2-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.handle_error/1","doc":"Handles various error types and converts them to standardized JSON-RPC errors.","ref":"MCPEx.Protocol.Errors.html#handle_error/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.handle_error/1","doc":"* `error` - The error to handle","ref":"MCPEx.Protocol.Errors.html#handle_error/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.handle_error/1","doc":"* `{:error, map()}` - Standardized error map","ref":"MCPEx.Protocol.Errors.html#handle_error/1-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.internal_error/1","doc":"Creates an internal error object.","ref":"MCPEx.Protocol.Errors.html#internal_error/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.internal_error/1","doc":"* `details` - Optional details about the internal error","ref":"MCPEx.Protocol.Errors.html#internal_error/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.internal_error/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#internal_error/1-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.invalid_params/1","doc":"Creates an invalid params error object.","ref":"MCPEx.Protocol.Errors.html#invalid_params/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.invalid_params/1","doc":"* `details` - Details about the invalid parameters","ref":"MCPEx.Protocol.Errors.html#invalid_params/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.invalid_params/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#invalid_params/1-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.invalid_request/1","doc":"Creates an invalid request error object.","ref":"MCPEx.Protocol.Errors.html#invalid_request/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.invalid_request/1","doc":"* `message` - Optional custom error message","ref":"MCPEx.Protocol.Errors.html#invalid_request/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.invalid_request/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#invalid_request/1-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.method_not_found/1","doc":"Creates a method not found error object.","ref":"MCPEx.Protocol.Errors.html#method_not_found/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.method_not_found/1","doc":"* `method` - The method that was not found","ref":"MCPEx.Protocol.Errors.html#method_not_found/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.method_not_found/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#method_not_found/1-returns"},{"type":"function","title":"MCPEx.Protocol.Errors.parse_error/1","doc":"Creates a parse error object.","ref":"MCPEx.Protocol.Errors.html#parse_error/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.parse_error/1","doc":"* `message` - Optional custom error message","ref":"MCPEx.Protocol.Errors.html#parse_error/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.parse_error/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#parse_error/1-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Errors.parse_error/1","doc":"iex> MCPEx.Protocol.Errors.parse_error()\n    %{code: -32700, message: \"Parse error\"}","ref":"MCPEx.Protocol.Errors.html#parse_error/1-examples"},{"type":"function","title":"MCPEx.Protocol.Errors.request_cancelled/1","doc":"Creates a request cancelled error object.","ref":"MCPEx.Protocol.Errors.html#request_cancelled/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Errors.request_cancelled/1","doc":"* `details` - Details about the cancellation","ref":"MCPEx.Protocol.Errors.html#request_cancelled/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Errors.request_cancelled/1","doc":"* `map()` - The error object","ref":"MCPEx.Protocol.Errors.html#request_cancelled/1-returns"},{"type":"module","title":"MCPEx.Protocol.JsonRpc","doc":"JSON-RPC 2.0 protocol implementation for MCP client.\n\nThis module handles encoding and decoding of JSON-RPC 2.0 messages according to the\nModel Context Protocol specification. It supports both individual messages and batches\nof messages as defined in the JSON-RPC 2.0 specification.","ref":"MCPEx.Protocol.JsonRpc.html"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.create_initialize_request/3","doc":"Creates an initialize request message as defined in the MCP specification.","ref":"MCPEx.Protocol.JsonRpc.html#create_initialize_request/3"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.create_initialize_request/3","doc":"* `client_info` - Information about the client\n* `protocol_version` - Protocol version to use\n* `capabilities` - Client capabilities","ref":"MCPEx.Protocol.JsonRpc.html#create_initialize_request/3-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.create_initialize_request/3","doc":"* `map()` - The initialize request message","ref":"MCPEx.Protocol.JsonRpc.html#create_initialize_request/3-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.decode_batch/1","doc":"Decodes a JSON-RPC batch message.","ref":"MCPEx.Protocol.JsonRpc.html#decode_batch/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.decode_batch/1","doc":"* `json` - The JSON-RPC batch to decode","ref":"MCPEx.Protocol.JsonRpc.html#decode_batch/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.decode_batch/1","doc":"* `{:ok, [map()]}` - The decoded messages\n* `{:error, reason}` - Error decoding the messages","ref":"MCPEx.Protocol.JsonRpc.html#decode_batch/1-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.decode_message/1","doc":"Decodes a JSON-RPC message.","ref":"MCPEx.Protocol.JsonRpc.html#decode_message/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.decode_message/1","doc":"* `json` - The JSON-RPC message to decode","ref":"MCPEx.Protocol.JsonRpc.html#decode_message/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.decode_message/1","doc":"* `{:ok, map()}` - The decoded message\n* `{:error, reason}` - Error decoding the message","ref":"MCPEx.Protocol.JsonRpc.html#decode_message/1-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.encode_batch/1","doc":"Encodes a batch of JSON-RPC messages.","ref":"MCPEx.Protocol.JsonRpc.html#encode_batch/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.encode_batch/1","doc":"* `messages` - A list of JSON-RPC messages (requests, notifications, responses)","ref":"MCPEx.Protocol.JsonRpc.html#encode_batch/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.encode_batch/1","doc":"* `String.t()` - The encoded JSON-RPC batch","ref":"MCPEx.Protocol.JsonRpc.html#encode_batch/1-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.encode_error/2","doc":"Encodes a JSON-RPC error response.","ref":"MCPEx.Protocol.JsonRpc.html#encode_error/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.encode_error/2","doc":"* `id` - The request ID\n* `error` - The error details","ref":"MCPEx.Protocol.JsonRpc.html#encode_error/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.encode_error/2","doc":"* `String.t()` - The encoded JSON-RPC error response","ref":"MCPEx.Protocol.JsonRpc.html#encode_error/2-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.encode_notification/2","doc":"Encodes a JSON-RPC notification.","ref":"MCPEx.Protocol.JsonRpc.html#encode_notification/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.encode_notification/2","doc":"* `method` - The method name to call\n* `params` - The parameters to pass to the method","ref":"MCPEx.Protocol.JsonRpc.html#encode_notification/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.encode_notification/2","doc":"* `String.t()` - The encoded JSON-RPC notification","ref":"MCPEx.Protocol.JsonRpc.html#encode_notification/2-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.encode_request/3","doc":"Encodes a JSON-RPC request.","ref":"MCPEx.Protocol.JsonRpc.html#encode_request/3"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.encode_request/3","doc":"* `id` - The request ID\n* `method` - The method name to call\n* `params` - The parameters to pass to the method","ref":"MCPEx.Protocol.JsonRpc.html#encode_request/3-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.encode_request/3","doc":"* `String.t()` - The encoded JSON-RPC request","ref":"MCPEx.Protocol.JsonRpc.html#encode_request/3-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.encode_response/2","doc":"Encodes a JSON-RPC response.","ref":"MCPEx.Protocol.JsonRpc.html#encode_response/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.encode_response/2","doc":"* `id` - The request ID\n* `result` - The result of the request","ref":"MCPEx.Protocol.JsonRpc.html#encode_response/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.encode_response/2","doc":"* `String.t()` - The encoded JSON-RPC response","ref":"MCPEx.Protocol.JsonRpc.html#encode_response/2-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.generate_id/0","doc":"Generates a unique request ID.","ref":"MCPEx.Protocol.JsonRpc.html#generate_id/0"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.generate_id/0","doc":"* `integer()` - A unique request ID","ref":"MCPEx.Protocol.JsonRpc.html#generate_id/0-returns"},{"type":"function","title":"MCPEx.Protocol.JsonRpc.validate_message/1","doc":"Validates a decoded JSON-RPC message.","ref":"MCPEx.Protocol.JsonRpc.html#validate_message/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.JsonRpc.validate_message/1","doc":"* `message` - The decoded message to validate","ref":"MCPEx.Protocol.JsonRpc.html#validate_message/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.JsonRpc.validate_message/1","doc":"* `{:ok, map()}` - The validated message\n* `{:error, reason}` - Invalid message","ref":"MCPEx.Protocol.JsonRpc.html#validate_message/1-returns"},{"type":"module","title":"MCPEx.Protocol.Lifecycle","doc":"Manages the MCP protocol lifecycle.\n\nThis module handles the initialization, operation, and shutdown\nphases of the protocol interaction, focusing on:\n\n1. Client and server initialization with capability negotiation\n2. Protocol version compatibility checking\n3. Required capability verification\n4. Handling the protocol state transitions","ref":"MCPEx.Protocol.Lifecycle.html"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.check_required_capabilities/2","doc":"Checks that required capabilities are available in the server capabilities.","ref":"MCPEx.Protocol.Lifecycle.html#check_required_capabilities/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.Lifecycle.check_required_capabilities/2","doc":"* `server_capabilities` - The server's capabilities\n* `required_capabilities` - List of required capabilities","ref":"MCPEx.Protocol.Lifecycle.html#check_required_capabilities/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.check_required_capabilities/2","doc":"* `:ok` - All required capabilities are available\n* `{:error, reason}` - Some required capabilities are missing","ref":"MCPEx.Protocol.Lifecycle.html#check_required_capabilities/2-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Lifecycle.check_required_capabilities/2","doc":"Simple capability list:\n\n    iex> server_capabilities = %{resources: %{}, tools: %{}}\n    iex> Lifecycle.check_required_capabilities(server_capabilities, [:resources])\n    :ok\n    \nWith sub-capability requirements:\n\n    iex> server_capabilities = %{resources: %{subscribe: true}}\n    iex> Lifecycle.check_required_capabilities(server_capabilities, [resources: [:subscribe]])\n    :ok","ref":"MCPEx.Protocol.Lifecycle.html#check_required_capabilities/2-examples"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.create_initialize_request/3","doc":"Creates an initialization request according to the MCP specification.","ref":"MCPEx.Protocol.Lifecycle.html#create_initialize_request/3"},{"type":"function","title":"Parameters - MCPEx.Protocol.Lifecycle.create_initialize_request/3","doc":"* `client_info` - Information about the client (name, version)\n* `capabilities` - The client's capabilities\n* `protocol_version` - The protocol version to use (defaults to latest)","ref":"MCPEx.Protocol.Lifecycle.html#create_initialize_request/3-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.create_initialize_request/3","doc":"* `InitializeRequest.t()` - The initialization request struct","ref":"MCPEx.Protocol.Lifecycle.html#create_initialize_request/3-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Lifecycle.create_initialize_request/3","doc":"iex> Lifecycle.create_initialize_request(\n    ...>   %{name: \"TestClient\", version: \"1.0.0\"},\n    ...>   %{sampling: %{}}\n    ...> )","ref":"MCPEx.Protocol.Lifecycle.html#create_initialize_request/3-examples"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.create_initialized_notification/0","doc":"Creates an initialized notification according to the MCP specification.","ref":"MCPEx.Protocol.Lifecycle.html#create_initialized_notification/0"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.create_initialized_notification/0","doc":"* `InitializedNotification.t()` - The initialized notification struct","ref":"MCPEx.Protocol.Lifecycle.html#create_initialized_notification/0-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Lifecycle.create_initialized_notification/0","doc":"iex> Lifecycle.create_initialized_notification()\n    %InitializedNotification{}","ref":"MCPEx.Protocol.Lifecycle.html#create_initialized_notification/0-examples"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.process_initialize_response/1","doc":"Processes an initialization response from the server.","ref":"MCPEx.Protocol.Lifecycle.html#process_initialize_response/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Lifecycle.process_initialize_response/1","doc":"* `response` - The server's response to the initialization request","ref":"MCPEx.Protocol.Lifecycle.html#process_initialize_response/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.process_initialize_response/1","doc":"* `{:ok, InitializeResult.t()}` - Successful initialization\n* `{:error, reason}` - Error during initialization","ref":"MCPEx.Protocol.Lifecycle.html#process_initialize_response/1-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Lifecycle.process_initialize_response/1","doc":"iex> response = %{\n    ...>   jsonrpc: \"2.0\", id: 1, \n    ...>   result: %{\n    ...>     serverInfo: %{name: \"Server\", version: \"1.0\"},\n    ...>     protocolVersion: \"2025-03-26\",\n    ...>     capabilities: %{}\n    ...>   }\n    ...> }\n    iex> {:ok, result} = Lifecycle.process_initialize_response(response)","ref":"MCPEx.Protocol.Lifecycle.html#process_initialize_response/1-examples"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.process_server_capabilities/1","doc":"Processes server capabilities from the initialize response.","ref":"MCPEx.Protocol.Lifecycle.html#process_server_capabilities/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Lifecycle.process_server_capabilities/1","doc":"* `capabilities` - The raw server capabilities map","ref":"MCPEx.Protocol.Lifecycle.html#process_server_capabilities/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.process_server_capabilities/1","doc":"* `{:ok, map()}` - Processed capabilities\n* `{:error, reason}` - Error processing capabilities","ref":"MCPEx.Protocol.Lifecycle.html#process_server_capabilities/1-returns"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.validate_protocol_version/1","doc":"Validates that a protocol version is supported by the client.","ref":"MCPEx.Protocol.Lifecycle.html#validate_protocol_version/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Lifecycle.validate_protocol_version/1","doc":"* `version` - The protocol version to check","ref":"MCPEx.Protocol.Lifecycle.html#validate_protocol_version/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.validate_protocol_version/1","doc":"* `:ok` - The version is supported\n* `{:error, reason}` - The version is not supported","ref":"MCPEx.Protocol.Lifecycle.html#validate_protocol_version/1-returns"},{"type":"function","title":"MCPEx.Protocol.Lifecycle.validate_server_initialization/1","doc":"Validates a server initialization response.","ref":"MCPEx.Protocol.Lifecycle.html#validate_server_initialization/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Lifecycle.validate_server_initialization/1","doc":"* `response` - The server's response to the initialization request","ref":"MCPEx.Protocol.Lifecycle.html#validate_server_initialization/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Lifecycle.validate_server_initialization/1","doc":"* `:ok` - The initialization is valid\n* `{:error, reason}` - There are issues with the initialization","ref":"MCPEx.Protocol.Lifecycle.html#validate_server_initialization/1-returns"},{"type":"module","title":"MCPEx.Protocol.Schemas","doc":"JSON Schema validation for MCP protocol messages.\n\nThis module provides validation of incoming and outgoing messages\nagainst the MCP protocol specification schemas.","ref":"MCPEx.Protocol.Schemas.html"},{"type":"function","title":"MCPEx.Protocol.Schemas.detect_type/1","doc":"Detects the message type based on its structure.","ref":"MCPEx.Protocol.Schemas.html#detect_type/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Schemas.detect_type/1","doc":"* `message` - The message to analyze","ref":"MCPEx.Protocol.Schemas.html#detect_type/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Schemas.detect_type/1","doc":"* `{:ok, type}` - Successfully detected the message type\n* `{:error, reason}` - Unable to determine the message type","ref":"MCPEx.Protocol.Schemas.html#detect_type/1-returns"},{"type":"function","title":"MCPEx.Protocol.Schemas.get_resolved_schema/0","doc":"Gets the resolved JSON schema for MCP protocol validation.","ref":"MCPEx.Protocol.Schemas.html#get_resolved_schema/0"},{"type":"function","title":"Returns - MCPEx.Protocol.Schemas.get_resolved_schema/0","doc":"* `{:ok, map()}` - The resolved schema\n* `{:error, reason}` - Failed to get the schema","ref":"MCPEx.Protocol.Schemas.html#get_resolved_schema/0-returns"},{"type":"function","title":"MCPEx.Protocol.Schemas.validate/2","doc":"Validates a message against a specific schema definition.","ref":"MCPEx.Protocol.Schemas.html#validate/2"},{"type":"function","title":"Parameters - MCPEx.Protocol.Schemas.validate/2","doc":"* `message` - The message to validate\n* `schema_type` - The name of the schema type (e.g., \"InitializeRequest\")","ref":"MCPEx.Protocol.Schemas.html#validate/2-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Schemas.validate/2","doc":"* `:ok` - The message is valid\n* `{:error, reason}` - The message is invalid","ref":"MCPEx.Protocol.Schemas.html#validate/2-returns"},{"type":"function","title":"MCPEx.Protocol.Schemas.validate_initialize_request/1","doc":"Validates an initialize request message.","ref":"MCPEx.Protocol.Schemas.html#validate_initialize_request/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Schemas.validate_initialize_request/1","doc":"* `message` - The initialize request message to validate","ref":"MCPEx.Protocol.Schemas.html#validate_initialize_request/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Schemas.validate_initialize_request/1","doc":"* `:ok` - The message is valid\n* `{:error, reason}` - The message is invalid","ref":"MCPEx.Protocol.Schemas.html#validate_initialize_request/1-returns"},{"type":"function","title":"MCPEx.Protocol.Schemas.validate_initialize_result/1","doc":"Validates an initialize response message.","ref":"MCPEx.Protocol.Schemas.html#validate_initialize_result/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Schemas.validate_initialize_result/1","doc":"* `message` - The initialize response message to validate","ref":"MCPEx.Protocol.Schemas.html#validate_initialize_result/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Schemas.validate_initialize_result/1","doc":"* `:ok` - The message is valid\n* `{:error, reason}` - The message is invalid","ref":"MCPEx.Protocol.Schemas.html#validate_initialize_result/1-returns"},{"type":"function","title":"MCPEx.Protocol.Schemas.validate_message/1","doc":"Validates a message by detecting its type and then validating against the appropriate schema.","ref":"MCPEx.Protocol.Schemas.html#validate_message/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Schemas.validate_message/1","doc":"* `message` - The message to validate","ref":"MCPEx.Protocol.Schemas.html#validate_message/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Schemas.validate_message/1","doc":"* `:ok` - The message is valid\n* `{:error, reason}` - The message is invalid","ref":"MCPEx.Protocol.Schemas.html#validate_message/1-returns"},{"type":"module","title":"MCPEx.Protocol.Types","doc":"Type definitions for MCP protocol messages.\n\nThis module provides Elixir structs representing the various\nmessage types in the MCP protocol, along with conversion functions\nbetween structs and JSON-RPC maps.","ref":"MCPEx.Protocol.Types.html"},{"type":"function","title":"MCPEx.Protocol.Types.detect_message_type/1","doc":"Detects the message type based on JSON-RPC message structure.","ref":"MCPEx.Protocol.Types.html#detect_message_type/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Types.detect_message_type/1","doc":"* `map` - The JSON-RPC message to analyze","ref":"MCPEx.Protocol.Types.html#detect_message_type/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Types.detect_message_type/1","doc":"* `{:ok, module()}` - The detected struct type\n* `{:error, String.t()}` - Error message if the type is unknown","ref":"MCPEx.Protocol.Types.html#detect_message_type/1-returns"},{"type":"function","title":"MCPEx.Protocol.Types.from_map/1","doc":"Converts a JSON-RPC map to the appropriate protocol struct.","ref":"MCPEx.Protocol.Types.html#from_map/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Types.from_map/1","doc":"* `map` - The JSON-RPC map to convert","ref":"MCPEx.Protocol.Types.html#from_map/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Types.from_map/1","doc":"* `{:ok, struct()}` - The converted protocol struct\n* `{:error, String.t()}` - Error message if the map doesn't match a known type","ref":"MCPEx.Protocol.Types.html#from_map/1-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Types.from_map/1","doc":"iex> map = %{jsonrpc: \"2.0\", id: 1, method: \"initialize\", params: %{}}\n    iex> {:ok, request} = MCPEx.Protocol.Types.from_map(map)\n    iex> request.__struct__\n    MCPEx.Protocol.Types.InitializeRequest","ref":"MCPEx.Protocol.Types.html#from_map/1-examples"},{"type":"type","title":"MCPEx.Protocol.Types.message/0","doc":"","ref":"MCPEx.Protocol.Types.html#t:message/0"},{"type":"function","title":"MCPEx.Protocol.Types.to_map/1","doc":"Converts a protocol struct to a JSON-RPC map representation.","ref":"MCPEx.Protocol.Types.html#to_map/1"},{"type":"function","title":"Parameters - MCPEx.Protocol.Types.to_map/1","doc":"* `struct` - The protocol struct to convert","ref":"MCPEx.Protocol.Types.html#to_map/1-parameters"},{"type":"function","title":"Returns - MCPEx.Protocol.Types.to_map/1","doc":"* `map()` - The JSON-RPC map","ref":"MCPEx.Protocol.Types.html#to_map/1-returns"},{"type":"function","title":"Examples - MCPEx.Protocol.Types.to_map/1","doc":"iex> request = %InitializeRequest{id: 1, client_info: %{name: \"client\"}}\n    iex> MCPEx.Protocol.Types.to_map(request)\n    %{jsonrpc: \"2.0\", id: 1, method: \"initialize\", params: %{clientInfo: %{name: \"client\"}}}","ref":"MCPEx.Protocol.Types.html#to_map/1-examples"},{"type":"module","title":"MCPEx.Protocol.Types.InitializeRequest","doc":"Represents an initialize request","ref":"MCPEx.Protocol.Types.InitializeRequest.html"},{"type":"type","title":"MCPEx.Protocol.Types.InitializeRequest.t/0","doc":"","ref":"MCPEx.Protocol.Types.InitializeRequest.html#t:t/0"},{"type":"module","title":"MCPEx.Protocol.Types.InitializeResult","doc":"Represents an initialize response","ref":"MCPEx.Protocol.Types.InitializeResult.html"},{"type":"type","title":"MCPEx.Protocol.Types.InitializeResult.t/0","doc":"","ref":"MCPEx.Protocol.Types.InitializeResult.html#t:t/0"},{"type":"module","title":"MCPEx.Protocol.Types.InitializedNotification","doc":"Represents an initialized notification","ref":"MCPEx.Protocol.Types.InitializedNotification.html"},{"type":"type","title":"MCPEx.Protocol.Types.InitializedNotification.t/0","doc":"","ref":"MCPEx.Protocol.Types.InitializedNotification.html#t:t/0"},{"type":"module","title":"MCPEx.Protocol.Types.ReadResourceRequest","doc":"Represents a resources/read request","ref":"MCPEx.Protocol.Types.ReadResourceRequest.html"},{"type":"type","title":"MCPEx.Protocol.Types.ReadResourceRequest.t/0","doc":"","ref":"MCPEx.Protocol.Types.ReadResourceRequest.html#t:t/0"},{"type":"module","title":"MCPEx.Protocol.Types.ResourceContent","doc":"Represents content types that can be sent in resources or\nother messages. This includes text, images, audio, or embedded \nresources.","ref":"MCPEx.Protocol.Types.ResourceContent.html"},{"type":"function","title":"MCPEx.Protocol.Types.ResourceContent.audio/2","doc":"Creates an audio content object","ref":"MCPEx.Protocol.Types.ResourceContent.html#audio/2"},{"type":"function","title":"MCPEx.Protocol.Types.ResourceContent.image/2","doc":"Creates an image content object","ref":"MCPEx.Protocol.Types.ResourceContent.html#image/2"},{"type":"function","title":"MCPEx.Protocol.Types.ResourceContent.resource/1","doc":"Creates a resource reference","ref":"MCPEx.Protocol.Types.ResourceContent.html#resource/1"},{"type":"type","title":"MCPEx.Protocol.Types.ResourceContent.t/0","doc":"","ref":"MCPEx.Protocol.Types.ResourceContent.html#t:t/0"},{"type":"function","title":"MCPEx.Protocol.Types.ResourceContent.text/1","doc":"Creates a text content object","ref":"MCPEx.Protocol.Types.ResourceContent.html#text/1"},{"type":"module","title":"MCPEx.Transport","doc":"Defines the behavior for MCP transport implementations.\n\nThe transport layer handles the communication between the client and server,\nproviding abstraction over different transport mechanisms (stdio, HTTP, etc.).","ref":"MCPEx.Transport.html"},{"type":"function","title":"MCPEx.Transport.close/1","doc":"Closes the transport connection.","ref":"MCPEx.Transport.html#close/1"},{"type":"function","title":"Parameters - MCPEx.Transport.close/1","doc":"* `transport` - The transport to close","ref":"MCPEx.Transport.html#close/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.close/1","doc":"* `:ok` - The connection was closed successfully\n* `{:error, reason}` - Failed to close the connection","ref":"MCPEx.Transport.html#close/1-returns"},{"type":"function","title":"MCPEx.Transport.create/2","doc":"Creates a new transport instance.","ref":"MCPEx.Transport.html#create/2"},{"type":"function","title":"Parameters - MCPEx.Transport.create/2","doc":"* `transport_type` - The type of transport to create (:stdio, :http, :test)\n* `options` - Options specific to the transport type","ref":"MCPEx.Transport.html#create/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.create/2","doc":"* `{:ok, pid}` - The transport was created successfully\n* `{:error, reason}` - Failed to create the transport","ref":"MCPEx.Transport.html#create/2-returns"},{"type":"function","title":"MCPEx.Transport.send/2","doc":"Sends a message to the server.","ref":"MCPEx.Transport.html#send/2"},{"type":"function","title":"Parameters - MCPEx.Transport.send/2","doc":"* `transport` - The transport process\n* `message` - The message to send","ref":"MCPEx.Transport.html#send/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.send/2","doc":"* `:ok` - The message was sent successfully\n* `{:error, reason}` - Failed to send the message","ref":"MCPEx.Transport.html#send/2-returns"},{"type":"module","title":"MCPEx.Transport.Http","doc":"Implementation of the MCP transport over HTTP with Server-Sent Events (SSE).\n\nThis transport follows the Model Context Protocol (MCP) HTTP transport specification:\n\n1. **SSE Connection**: The client connects to an SSE endpoint (e.g., `https://example.com/sse`) to receive server events\n2. **Message Endpoint Notification**: The server sends a special `endpoint` event via SSE with the following format:\n   ```\n   event: endpoint\n   data: /messages?sessionId=abc123\n   ```\n3. **Message Endpoint Construction**: The client constructs the full message endpoint URL by using the same origin as the SSE connection and appending the received path and query string (e.g., `https://example.com/messages?sessionId=abc123`)\n4. **Message Exchange**: The client sends all JSON-RPC messages as HTTP POST requests to this constructed message endpoint\n\nThis approach allows the server to dynamically configure where messages should be sent,\nsupporting load balancing and session routing while maintaining the same origin. The transport manages this by:\n\n- Establishing and maintaining the SSE connection\n- Processing the endpoint event to configure the message endpoint\n- Sending all client messages to the configured endpoint\n- Handling server messages received via the SSE stream\n\nThis transport follows Req's API patterns, making it easy to test with custom HTTP clients.","ref":"MCPEx.Transport.Http.html"},{"type":"module","title":"Testing - MCPEx.Transport.Http","doc":"There are two main approaches for testing code that uses this transport:\n\n1. Using a custom HTTP client (recommended):\n```elixir\n# Define a mock HTTP client for testing\ndefmodule MockHttpClient do\n  def post(_url, json_data, _headers, _timeout) do\n    # Verify request data\n    assert json_data.method == \"test\"\n    \n    # Return a mock response\n    {:ok, %{status: 200, body: %{\"result\" => \"success\"}}}\n  end\nend\n\n# Start the transport with the mock client\n{:ok, transport} = Http.start_link(\n  url: \"https://example.com/sse\",\n  http_client: MockHttpClient\n)\n\n# Simulate the SSE message endpoint notification\n# Note: only the path is sent, the client constructs the full URL\nsend(transport, {:sse_endpoint, \"/messages?sessionId=abc123\"})\n\n# Test sending a message\n{:ok, response} = Http.send_message(transport, %{method: \"test\"})\nassert response.status == 200\nassert response.body[\"result\"] == \"success\"\n```\n\n2. Using Req.Test (for testing without process boundaries):\n```elixir\n# Set up Req.Test\nReq.Test.stub(:test_stub, fn conn ->\n  assert conn.body[\"method\"] == \"test\"\n  %{conn | status: 200, body: %{\"result\" => \"success\"}}\nend)\n\n# Create a request with the test plug\nreq = Http.new(\n  url: \"https://example.com/sse\",\n  plug: {Req.Test, :test_stub}\n)\n\n# This approach works for direct Req calls but has limitations\n# with the GenServer process boundary in the transport\n```\n\nSee product/docs/010.01.req-usage.md for more detailed examples and patterns.","ref":"MCPEx.Transport.Http.html#module-testing"},{"type":"function","title":"MCPEx.Transport.Http.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MCPEx.Transport.Http.html#child_spec/1"},{"type":"function","title":"MCPEx.Transport.Http.close/1","doc":"Closes the transport connection.","ref":"MCPEx.Transport.Http.html#close/1"},{"type":"function","title":"Parameters - MCPEx.Transport.Http.close/1","doc":"* `pid` - The transport process","ref":"MCPEx.Transport.Http.html#close/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Http.close/1","doc":"* `:ok` - The connection was closed successfully","ref":"MCPEx.Transport.Http.html#close/1-returns"},{"type":"function","title":"MCPEx.Transport.Http.new/1","doc":"Creates a new HTTP transport request configuration.","ref":"MCPEx.Transport.Http.html#new/1"},{"type":"function","title":"Options - MCPEx.Transport.Http.new/1","doc":"* `:url` - The URL of the MCP server SSE endpoint (required)\n* `:headers` - Additional HTTP headers to include in requests\n* `:timeout` - Request timeout in milliseconds (default: 30000)\n* `:http_client` - Optional custom HTTP client module for testing (see example in tests)\n* Plus any other options from Req.new/1","ref":"MCPEx.Transport.Http.html#new/1-options"},{"type":"function","title":"Returns - MCPEx.Transport.Http.new/1","doc":"* `%Req.Request{}` - A request configuration for the HTTP transport","ref":"MCPEx.Transport.Http.html#new/1-returns"},{"type":"function","title":"Examples - MCPEx.Transport.Http.new/1","doc":"iex> req = MCPEx.Transport.Http.new(url: \"https://example.com/sse\")\n    %Req.Request{}","ref":"MCPEx.Transport.Http.html#new/1-examples"},{"type":"function","title":"MCPEx.Transport.Http.send_message/2","doc":"Sends a message to the server via HTTP POST.","ref":"MCPEx.Transport.Http.html#send_message/2"},{"type":"function","title":"Parameters - MCPEx.Transport.Http.send_message/2","doc":"* `pid` - The transport process\n* `message` - The message to send (string or map)","ref":"MCPEx.Transport.Http.html#send_message/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Http.send_message/2","doc":"* `{:ok, response}` - The message was sent successfully with the full response\n* `{:error, reason}` - Failed to send the message","ref":"MCPEx.Transport.Http.html#send_message/2-returns"},{"type":"function","title":"Examples - MCPEx.Transport.Http.send_message/2","doc":"iex> Http.send_message(pid, ~s({\"jsonrpc\":\"2.0\",\"method\":\"test\"}))\n    {:ok, %{status: 200, body: %{...}}}\n\n    iex> Http.send_message(pid, %{jsonrpc: \"2.0\", method: \"test\"})\n    {:ok, %{status: 200, body: %{...}}}","ref":"MCPEx.Transport.Http.html#send_message/2-examples"},{"type":"function","title":"MCPEx.Transport.Http.start_link/1","doc":"Starts a new HTTP transport as a linked process.","ref":"MCPEx.Transport.Http.html#start_link/1"},{"type":"function","title":"Options - MCPEx.Transport.Http.start_link/1","doc":"* `:request` - A Req.Request struct created with Http.new/1\n* `:http_client` - Optional custom HTTP client module for testing\n* Or any option accepted by Http.new/1","ref":"MCPEx.Transport.Http.html#start_link/1-options"},{"type":"function","title":"Returns - MCPEx.Transport.Http.start_link/1","doc":"* `{:ok, pid}` - The transport was started successfully\n* `{:error, reason}` - Failed to start the transport","ref":"MCPEx.Transport.Http.html#start_link/1-returns"},{"type":"function","title":"Examples - MCPEx.Transport.Http.start_link/1","doc":"# Using Http.new\n    iex> req = MCPEx.Transport.Http.new(url: \"https://example.com/sse\")\n    iex> {:ok, pid} = MCPEx.Transport.Http.start_link(request: req)\n\n    # Or directly with options\n    iex> {:ok, pid} = MCPEx.Transport.Http.start_link(url: \"https://example.com/sse\")","ref":"MCPEx.Transport.Http.html#start_link/1-examples"},{"type":"module","title":"MCPEx.Transport.Stdio","doc":"Implementation of the MCP transport over standard I/O.\n\nThis transport spawns an external process and communicates with it over\nstandard input/output pipes. It handles encoding/decoding of messages\nand manages the lifecycle of the external process.","ref":"MCPEx.Transport.Stdio.html"},{"type":"function","title":"MCPEx.Transport.Stdio.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MCPEx.Transport.Stdio.html#child_spec/1"},{"type":"function","title":"MCPEx.Transport.Stdio.close/1","doc":"Closes the transport connection.","ref":"MCPEx.Transport.Stdio.html#close/1"},{"type":"function","title":"Returns - MCPEx.Transport.Stdio.close/1","doc":"* `:ok` - The connection was closed successfully","ref":"MCPEx.Transport.Stdio.html#close/1-returns"},{"type":"function","title":"MCPEx.Transport.Stdio.send_message/2","doc":"Sends a message to the process.","ref":"MCPEx.Transport.Stdio.html#send_message/2"},{"type":"function","title":"Parameters - MCPEx.Transport.Stdio.send_message/2","doc":"* `message` - The message to send","ref":"MCPEx.Transport.Stdio.html#send_message/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Stdio.send_message/2","doc":"* `:ok` - The message was sent successfully\n* `{:error, reason}` - Failed to send the message","ref":"MCPEx.Transport.Stdio.html#send_message/2-returns"},{"type":"function","title":"MCPEx.Transport.Stdio.start_link/1","doc":"Starts a new stdio transport as a linked process.","ref":"MCPEx.Transport.Stdio.html#start_link/1"},{"type":"function","title":"Options - MCPEx.Transport.Stdio.start_link/1","doc":"* `:command` - The command to execute (required)\n* `:args` - List of command arguments (default: [])\n* `:cd` - Working directory for the command (default: current directory)\n* `:env` - Environment variables for the command (default: current environment)","ref":"MCPEx.Transport.Stdio.html#start_link/1-options"},{"type":"function","title":"Returns - MCPEx.Transport.Stdio.start_link/1","doc":"* `{:ok, pid}` - The transport was started successfully\n* `{:error, reason}` - Failed to start the transport","ref":"MCPEx.Transport.Stdio.html#start_link/1-returns"},{"type":"module","title":"MCPEx.Transport.Test","doc":"Test implementation of the MCP transport for integration testing.\n\nThis transport operates fully in memory and connects to an MCPEx.Transport.TestServer\ninstance to simulate the server side of the protocol.","ref":"MCPEx.Transport.Test.html"},{"type":"module","title":"Features - MCPEx.Transport.Test","doc":"* In-memory communication with no external dependencies\n* Controllable message flow and timing\n* Observable message exchange for testing\n* Configurable error scenarios\n* Complete protocol simulation","ref":"MCPEx.Transport.Test.html#module-features"},{"type":"module","title":"Usage - MCPEx.Transport.Test","doc":"```elixir\n# Create a test server\nserver = MCPEx.Transport.TestServer.new(\n  capabilities: [:resources, :tools]\n)\n\n# Create a test transport connected to this server\n{:ok, transport} = MCPEx.Transport.Test.start_link(server: server)\n\n# Register a message receiver (optional)\nMCPEx.Transport.Test.register_message_receiver(transport, self())\n\n# Send a message and get response\n{:ok, response} = MCPEx.Transport.send(transport, json_encoded_message)\n```","ref":"MCPEx.Transport.Test.html#module-usage"},{"type":"function","title":"MCPEx.Transport.Test.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MCPEx.Transport.Test.html#child_spec/1"},{"type":"function","title":"MCPEx.Transport.Test.connected?/1","doc":"Checks if the transport is connected to the server.","ref":"MCPEx.Transport.Test.html#connected?/1"},{"type":"function","title":"Parameters - MCPEx.Transport.Test.connected?/1","doc":"* `transport` - The transport process","ref":"MCPEx.Transport.Test.html#connected?/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Test.connected?/1","doc":"* `true` or `false`","ref":"MCPEx.Transport.Test.html#connected?/1-returns"},{"type":"function","title":"MCPEx.Transport.Test.get_log/1","doc":"Gets the current message log for debugging/testing purposes.","ref":"MCPEx.Transport.Test.html#get_log/1"},{"type":"function","title":"Parameters - MCPEx.Transport.Test.get_log/1","doc":"* `transport` - The transport process","ref":"MCPEx.Transport.Test.html#get_log/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Test.get_log/1","doc":"* List of log entries","ref":"MCPEx.Transport.Test.html#get_log/1-returns"},{"type":"function","title":"MCPEx.Transport.Test.new/1","doc":"Creates a new test transport instance without starting a process.","ref":"MCPEx.Transport.Test.html#new/1"},{"type":"function","title":"Options - MCPEx.Transport.Test.new/1","doc":"* `:server` - The MCPEx.Transport.TestServer instance to connect to (required)\n* `:config` - Additional configuration options","ref":"MCPEx.Transport.Test.html#new/1-options"},{"type":"function","title":"Returns - MCPEx.Transport.Test.new/1","doc":"* A new TestTransport struct","ref":"MCPEx.Transport.Test.html#new/1-returns"},{"type":"function","title":"MCPEx.Transport.Test.record_session/1","doc":"Records a session of message exchanges for later replay.","ref":"MCPEx.Transport.Test.html#record_session/1"},{"type":"function","title":"Parameters - MCPEx.Transport.Test.record_session/1","doc":"* `fun` - Function to execute during recording","ref":"MCPEx.Transport.Test.html#record_session/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Test.record_session/1","doc":"* `{:ok, recording}` - The recorded session\n* `{:error, reason}` - Failed to record the session","ref":"MCPEx.Transport.Test.html#record_session/1-returns"},{"type":"function","title":"MCPEx.Transport.Test.register_message_receiver/2","doc":"Registers a process to receive message notifications.\n\nThe registered process will receive `{:message, message}` tuples\nfor all messages received from the server.","ref":"MCPEx.Transport.Test.html#register_message_receiver/2"},{"type":"function","title":"Parameters - MCPEx.Transport.Test.register_message_receiver/2","doc":"* `transport` - The transport process\n* `receiver` - The process to receive notifications","ref":"MCPEx.Transport.Test.html#register_message_receiver/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Test.register_message_receiver/2","doc":"* `:ok`","ref":"MCPEx.Transport.Test.html#register_message_receiver/2-returns"},{"type":"function","title":"MCPEx.Transport.Test.replay_session/1","doc":"Replays a previously recorded session.","ref":"MCPEx.Transport.Test.html#replay_session/1"},{"type":"function","title":"Parameters - MCPEx.Transport.Test.replay_session/1","doc":"* `recording` - Recording from record_session","ref":"MCPEx.Transport.Test.html#replay_session/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Test.replay_session/1","doc":"* `{:ok, transport}` - Transport for the replayed session\n* `{:error, reason}` - Failed to replay the session","ref":"MCPEx.Transport.Test.html#replay_session/1-returns"},{"type":"function","title":"MCPEx.Transport.Test.send_message/2","doc":"Sends a message to the server and waits for a response.","ref":"MCPEx.Transport.Test.html#send_message/2"},{"type":"function","title":"Parameters - MCPEx.Transport.Test.send_message/2","doc":"* `transport` - The transport process\n* `message` - The message to send, either as a map or JSON string","ref":"MCPEx.Transport.Test.html#send_message/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.Test.send_message/2","doc":"* `{:ok, response}` - The response from the server\n* `{:error, reason}` - Failed to send the message or receive a response","ref":"MCPEx.Transport.Test.html#send_message/2-returns"},{"type":"function","title":"MCPEx.Transport.Test.start_link/1","doc":"Starts a new test transport as a linked process.","ref":"MCPEx.Transport.Test.html#start_link/1"},{"type":"function","title":"Options - MCPEx.Transport.Test.start_link/1","doc":"* `:server` - The MCPEx.Transport.TestServer instance to connect to (required)\n* `:auto_connect` - Whether to automatically connect to the server (default: true)\n* `:config` - Additional configuration options","ref":"MCPEx.Transport.Test.html#start_link/1-options"},{"type":"function","title":"Returns - MCPEx.Transport.Test.start_link/1","doc":"* `{:ok, pid}` - The transport was started successfully\n* `{:error, reason}` - Failed to start the transport","ref":"MCPEx.Transport.Test.html#start_link/1-returns"},{"type":"type","title":"MCPEx.Transport.Test.t/0","doc":"","ref":"MCPEx.Transport.Test.html#t:t/0"},{"type":"module","title":"MCPEx.Transport.TestServer","doc":"Test server implementation to pair with MCPEx.Transport.Test.\n\nThis module simulates an MCP server for integration testing, allowing\ntests to verify client behavior without requiring an actual MCP server.","ref":"MCPEx.Transport.TestServer.html"},{"type":"module","title":"Features - MCPEx.Transport.TestServer","doc":"* Simulate server responses and notifications\n* Control message timing and failures\n* Test specific scenarios and protocol flows\n* Verify client requests match expectations\n* Support all MCP protocol features (resources, tools, prompts, sampling)","ref":"MCPEx.Transport.TestServer.html#module-features"},{"type":"module","title":"Usage - MCPEx.Transport.TestServer","doc":"```elixir\n# Create a test server with capabilities\nserver = MCPEx.Transport.TestServer.new(\n  capabilities: [:resources, :tools]\n)\n\n# Start a test transport connected to this server\n{:ok, transport} = MCPEx.Transport.Test.start_link(server: server)\n\n# Trigger server events\nMCPEx.Transport.TestServer.send_notification(server, %{\n  jsonrpc: \"2.0\",\n  method: \"notifications/resources/updated\"\n})\n\n# Verify client messages\nassert MCPEx.Transport.TestServer.last_message(server).method == \"initialize\"\n```","ref":"MCPEx.Transport.TestServer.html#module-usage"},{"type":"function","title":"MCPEx.Transport.TestServer.add_handler/3","doc":"Adds a custom handler for a specific method or response.","ref":"MCPEx.Transport.TestServer.html#add_handler/3"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.add_handler/3","doc":"* `server` - The server struct\n* `method` - The method name to handle\n* `handler_fn` - A function that takes the params and returns the result or an error","ref":"MCPEx.Transport.TestServer.html#add_handler/3-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.add_handler/3","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#add_handler/3-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.add_prompt/2","doc":"Adds a prompt to the server.","ref":"MCPEx.Transport.TestServer.html#add_prompt/2"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.add_prompt/2","doc":"* `server` - The server struct\n* `prompt` - The prompt to add","ref":"MCPEx.Transport.TestServer.html#add_prompt/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.add_prompt/2","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#add_prompt/2-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.add_resource/2","doc":"Adds a resource to the server.","ref":"MCPEx.Transport.TestServer.html#add_resource/2"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.add_resource/2","doc":"* `server` - The server struct\n* `resource` - The resource to add","ref":"MCPEx.Transport.TestServer.html#add_resource/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.add_resource/2","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#add_resource/2-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.add_tool/2","doc":"Adds a tool to the server.","ref":"MCPEx.Transport.TestServer.html#add_tool/2"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.add_tool/2","doc":"* `server` - The server struct\n* `tool` - The tool to add","ref":"MCPEx.Transport.TestServer.html#add_tool/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.add_tool/2","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#add_tool/2-returns"},{"type":"type","title":"MCPEx.Transport.TestServer.client_ref/0","doc":"","ref":"MCPEx.Transport.TestServer.html#t:client_ref/0"},{"type":"function","title":"MCPEx.Transport.TestServer.error/3","doc":"Creates a standard error response for use in handlers.","ref":"MCPEx.Transport.TestServer.html#error/3"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.error/3","doc":"* `error_type` - The type of error (atom like :invalid_params, :parse_error)\n* `message` - The error message\n* `data` - Optional additional data about the error","ref":"MCPEx.Transport.TestServer.html#error/3-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.error/3","doc":"* `{:error, code, message}` - Error tuple that can be returned from handlers","ref":"MCPEx.Transport.TestServer.html#error/3-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.get_recording/1","doc":"Gets the current recording for later replay.","ref":"MCPEx.Transport.TestServer.html#get_recording/1"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.get_recording/1","doc":"* `server` - The server struct","ref":"MCPEx.Transport.TestServer.html#get_recording/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.get_recording/1","doc":"* The recording data or nil if not in recording mode","ref":"MCPEx.Transport.TestServer.html#get_recording/1-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.last_message/1","doc":"Gets the last message received by the server.","ref":"MCPEx.Transport.TestServer.html#last_message/1"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.last_message/1","doc":"* `server` - The server struct","ref":"MCPEx.Transport.TestServer.html#last_message/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.last_message/1","doc":"* The last message or nil if none","ref":"MCPEx.Transport.TestServer.html#last_message/1-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.new/1","doc":"Creates a new test server instance.","ref":"MCPEx.Transport.TestServer.html#new/1"},{"type":"function","title":"Options - MCPEx.Transport.TestServer.new/1","doc":"* `:capabilities` - List of server capabilities (default: [])\n* `:scenario` - Name of the test scenario to run (default: nil)\n* `:delay` - Artificial delay in milliseconds for responses (default: 0)\n* `:error_rate` - Probability of simulated errors (0.0-1.0, default: 0.0)\n* `:resources` - Initial resources to serve (default: [])\n* `:tools` - Initial tools to serve (default: [])\n* `:prompts` - Initial prompts to serve (default: [])","ref":"MCPEx.Transport.TestServer.html#new/1-options"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.new/1","doc":"* A new TestServer struct","ref":"MCPEx.Transport.TestServer.html#new/1-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.process_message/3","doc":"Processes a message from a client.","ref":"MCPEx.Transport.TestServer.html#process_message/3"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.process_message/3","doc":"* `server` - The server struct\n* `client_ref` - Reference for the client sending the message\n* `message` - The message received from the client","ref":"MCPEx.Transport.TestServer.html#process_message/3-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.process_message/3","doc":"* `{response, updated_server}` - Response to send back and updated server","ref":"MCPEx.Transport.TestServer.html#process_message/3-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.received_message?/2","doc":"Checks if the server received a message matching the given criteria.","ref":"MCPEx.Transport.TestServer.html#received_message?/2"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.received_message?/2","doc":"* `server` - The server struct\n* `criteria` - Criteria to match (method, id, etc.)","ref":"MCPEx.Transport.TestServer.html#received_message?/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.received_message?/2","doc":"* `true` or `false`","ref":"MCPEx.Transport.TestServer.html#received_message?/2-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.received_messages/1","doc":"Gets all messages received by the server.","ref":"MCPEx.Transport.TestServer.html#received_messages/1"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.received_messages/1","doc":"* `server` - The server struct","ref":"MCPEx.Transport.TestServer.html#received_messages/1-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.received_messages/1","doc":"* List of messages in reverse chronological order","ref":"MCPEx.Transport.TestServer.html#received_messages/1-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.register_client/2","doc":"Registers a client with the server.","ref":"MCPEx.Transport.TestServer.html#register_client/2"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.register_client/2","doc":"* `server` - The server struct\n* `client_pid` - PID of the client transport process","ref":"MCPEx.Transport.TestServer.html#register_client/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.register_client/2","doc":"* `{ref, updated_server}` - The client reference and updated server","ref":"MCPEx.Transport.TestServer.html#register_client/2-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.request_sampling/3","doc":"Triggers a sampling request.","ref":"MCPEx.Transport.TestServer.html#request_sampling/3"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.request_sampling/3","doc":"* `server` - The server struct\n* `options` - Options for the sampling request\n* `client_ref` - Optional client reference (first client if nil)","ref":"MCPEx.Transport.TestServer.html#request_sampling/3-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.request_sampling/3","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#request_sampling/3-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.send_notification/3","doc":"Sends a notification message to a client.","ref":"MCPEx.Transport.TestServer.html#send_notification/3"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.send_notification/3","doc":"* `server` - The server struct\n* `notification` - The notification message to send\n* `client_ref` - Optional client reference (sends to all if nil)","ref":"MCPEx.Transport.TestServer.html#send_notification/3-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.send_notification/3","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#send_notification/3-returns"},{"type":"function","title":"MCPEx.Transport.TestServer.set_response_handler/3","doc":"Sets a custom response handler for a specific method.","ref":"MCPEx.Transport.TestServer.html#set_response_handler/3"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.set_response_handler/3","doc":"* `server` - The server struct\n* `method` - The method to handle\n* `handler` - Function that takes (server, message) and returns response","ref":"MCPEx.Transport.TestServer.html#set_response_handler/3-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.set_response_handler/3","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#set_response_handler/3-returns"},{"type":"type","title":"MCPEx.Transport.TestServer.t/0","doc":"","ref":"MCPEx.Transport.TestServer.html#t:t/0"},{"type":"function","title":"MCPEx.Transport.TestServer.unregister_client/2","doc":"Unregisters a client from the server.","ref":"MCPEx.Transport.TestServer.html#unregister_client/2"},{"type":"function","title":"Parameters - MCPEx.Transport.TestServer.unregister_client/2","doc":"* `server` - The server struct\n* `client_ref` - Reference for the client to unregister","ref":"MCPEx.Transport.TestServer.html#unregister_client/2-parameters"},{"type":"function","title":"Returns - MCPEx.Transport.TestServer.unregister_client/2","doc":"* Updated server struct","ref":"MCPEx.Transport.TestServer.html#unregister_client/2-returns"},{"type":"module","title":"MCPEx.Utils.CommandUtils","doc":"Utility functions for finding executables and executing commands through the shell.\nUsed by various parts of the library that need to interact with the system shell.","ref":"MCPEx.Utils.CommandUtils.html"},{"type":"function","title":"MCPEx.Utils.CommandUtils.execute_through_shell/1","doc":"Executes a command through the user's shell to ensure\nit has access to the full shell environment, including PATH from\nstartup files (.zshrc, .bashrc, etc).\n\nReturns {output, exit_code}","ref":"MCPEx.Utils.CommandUtils.html#execute_through_shell/1"},{"type":"function","title":"MCPEx.Utils.CommandUtils.find_executable/1","doc":"Find an executable in the PATH using the user's shell environment.\nReturns the full path to the executable or nil if not found.\nFalls back to system PATH search if shell environment fails.","ref":"MCPEx.Utils.CommandUtils.html#find_executable/1"},{"type":"function","title":"MCPEx.Utils.CommandUtils.get_user_shell/0","doc":"Gets the user's shell executable path and name.\nReturns a tuple of {shell_path, shell_name} where shell_path is the full path\nto the shell executable and shell_name is the base name (zsh, bash, etc).\nFalls back to /bin/bash if SHELL is not set.","ref":"MCPEx.Utils.CommandUtils.html#get_user_shell/0"},{"type":"function","title":"MCPEx.Utils.CommandUtils.to_charlist_if_found/1","doc":"Convert string to charlist for Port.open, returns nil if input is nil","ref":"MCPEx.Utils.CommandUtils.html#to_charlist_if_found/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}